@@ -19,10 +19,11 @@ from .options import Options
 from .Worker import Worker
 from .Fork import Fork
 from .ErrorMs import ErrorMs
-from .Util import validForksConf, getItemBy
+from .Util import validForksConf, getItemBy, Merge
 from .version_framework import version
 from http.server import BaseHTTPRequestHandler, HTTPServer
 from .types import TypesActions
+from .HttpResponse import HttpResponse
 
 if os.environ.get('KAFKA', None):
     from .ms_base import KafkaBase as BaseClass
@@ -427,9 +428,43 @@ class Microservices(BaseClass, ABC):
 
         except ErrorMs as error:
             span.set_tag(ext_tags.ERROR, True)
-            span.log_kv({'event': 'error', 'error.code': error.errorCode, 'error.message': self.___getErrorId(
-                error.errorCode), 'error.stack': traceback.format_exc()})
-            self.logs.error(self.___getErrorId(error.errorCode))
+
+            # Recuperar el registro del error
+            error_reg = self.___getErrorId(error.errorCode)
+
+            # Validar si es un diccionario
+            isDict = isinstance(error_reg, dict)
+
+            span.log_kv({
+                'event': 'error',
+                'error.code': error.errorCode,
+                'error.message': error_reg.get(
+                    'message', ""
+                ) if isDict else error_reg,
+                'error.stack': traceback.format_exc()
+            })
+
+            self.logs.error(error_reg.get(
+                'message', ""
+            ) if isDict else error_reg)
+
+            # Verificar si es un codigo de error ya especificado
+            if error.http_code is not None:
+                msg["response"]["data"]["response"] = HttpResponse(
+                    msg["response"]["data"]["response"],
+                    error.http_code,
+                    error.message,
+                    error.headers
+                )
+            else:
+                if isDict and "http_code" in error_reg:
+                    msg["response"]["data"]["response"] = HttpResponse(
+                        msg["response"]["data"]["response"],
+                        error_reg["http_code"],
+                        error_reg.get('message', ""),
+                        error.headers
+                    )
+
             # Validar si es async
             if not msg['metadata']['mtype'] == 'WITHOUT_RESPONSE':
                 self.__response(
@@ -460,6 +495,23 @@ class Microservices(BaseClass, ABC):
         # Diccionario de la respuesta
         Resp = {}
 
+        # Datos http.
+        HTTP = None
+
+        # Respuesta del servicio
+        RESPONSE_DATA = data["response"]["data"]["response"]
+
+        # Validar si data no es de tipo httpResponse
+        if (isinstance(RESPONSE_DATA, HttpResponse)):
+            http_data = data["response"]["data"]["response"]
+            HTTP = {
+                "code": http_data.code,
+                "headers": http_data.headers,
+                "message": http_data.message
+            }
+            # Reset de data
+            data["response"]["data"]["response"] = http_data.data
+
         # Crear una copia de la respuesta
         dataResponse = data
 
@@ -487,6 +539,18 @@ class Microservices(BaseClass, ABC):
 
         # Validar si ocurrio un error
         if errorCode < 0 or errorCode > 0:
+            # Mensaje del error
+            message_error = ""
+
+            # Registro del error
+            reg_error = self.___getErrorId(errorCode)
+
+            # Validar si es un string o un objeto de error
+            if isinstance(reg_error, dict):
+                message_error = reg_error.get('message', "")
+            else:
+                message_error = reg_error
+
             # Estructura del mensaje de error
             dataResponse['response'] = {
                 "data": {
@@ -494,8 +558,9 @@ class Microservices(BaseClass, ABC):
                         "hostname": self.__HOSTNAME,
                         "code": errorCode,
                         "errorCode": errorCode,
-                        # Regresar el mensaje que se proporciono desde el worker que se consumio
-                        "userMessage": self.___getErrorId(errorCode) if errorResp is None else errorResp.get('userMessage')
+                        # Regresar el mensaje que se proporciono desde el
+                        # worker que se consumio
+                        "userMessage": message_error if errorResp is None else errorResp.get('userMessage')
                     }
                 },
                 "meta": {
@@ -701,6 +766,21 @@ class Microservices(BaseClass, ABC):
         if nats_route is not None:
             Resp.update({"uuid": nats_route})
 
+        # Validar si se envia el objeto HTTP perzonalizado
+        if HTTP is not None:
+            Resp['metadata'].update({
+                "http": HTTP
+            })
+        else:
+            # Datos por default
+            Resp['metadata'].update({
+                "http": {
+                    "code": 500 if errorCode < 0 or errorCode > 0 else 200,
+                    "headers": {},
+                    "message": ""
+                }
+            })
+
         # Validar si es modeo DEBUG
         if self.__OPT.Debug and self.__fluentd_logs:
             self.logs.info("SALIDA [{}]".format(TOPIC_RESP), Resp)
@@ -743,23 +823,37 @@ class Microservices(BaseClass, ABC):
             ejecutara, segun los datos pasados en
             el metada.
         """
-        if meta.get('method', None) and (meta.get('uuid', None) and len(meta.get('uuid', '')) > 0):
-            if meta.get('method', None) == 'GET':  # Consultar un elemento
-                return self.__caller(self.__ACTIONS.get(TypesActions.GET))
-            elif meta.get('method', None) == 'DELETE':  # Eliminar el elemento
-                return self.__caller(self.__ACTIONS.get(TypesActions.DELETE))
-            elif meta.get('method', None) == 'PUT':  # Actualizar el elemento
-                return self.__caller(self.__ACTIONS.get(TypesActions.UPDATE))
-        elif meta.get('method', None) == 'GET':  # Lista de servicios
-            if self.__ACTIONS.get(TypesActions.LIST, None):
-                return self.__caller(self.__ACTIONS.get(TypesActions.LIST))
+        # Recuperar el metodo
+        METHOD = meta.get('method', None)
 
-        elif meta.get('method') == 'POST':  # Creacion de un nuevo elemento
-            if self.__ACTIONS.get(TypesActions.CREATE, None):
-                return self.__caller(self.__ACTIONS.get(TypesActions.CREATE))
+        # Metodo que se ejecutara
+        FNC = None
 
-        # Regresar el Listener por default
-        return self.__getListener()
+        # Seleccion del metodo
+        if METHOD:
+            if (meta.get('uuid', None) and len(meta.get('uuid', '')) > 0):
+                if METHOD == 'GET':  # Consultar un elemento
+                    FNC = self.__caller(self.__ACTIONS.get(TypesActions.GET))
+                elif METHOD == 'DELETE':  # Eliminar el elemento
+                    FNC = self.__caller(
+                        self.__ACTIONS.get(TypesActions.DELETE))
+                elif METHOD == 'PUT':  # Actualizar el elemento
+                    FNC = self.__caller(
+                        self.__ACTIONS.get(TypesActions.UPDATE))
+            elif METHOD == 'GET':  # Lista de servicios
+                if self.__ACTIONS.get(TypesActions.LIST, None):
+                    FNC = self.__caller(self.__ACTIONS.get(TypesActions.LIST))
+            elif METHOD == 'POST':  # Creacion de un nuevo elemento
+                if self.__ACTIONS.get(TypesActions.CREATE, None):
+                    FNC = self.__caller(
+                        self.__ACTIONS.get(TypesActions.CREATE))
+
+        # Validar si se recuper un metodo asignado
+        if FNC is not None:
+            return FNC
+        else:
+            # Regresar el Listener por default
+            return self.__getListener()
 
     def __initActions(self):
         """
